{% extends "wall/_base.html" %}

{% from "macros/box.html" import m_box_content, m_box_menu %}
{% from "macros/activity.html" import m_activities %}

{% block content %}
  {%- call m_box_content(_("Files")) %}

    {% for month, docs in grouped_docs %}
      <h2>{{ month }}</h2>

      {% for doc in docs %}
        <p>
          <span class="attachment-item">
            {{ doc.owner }} -

            <img src="{{ doc.icon }}" style="vertical-align: middle;"
                 alt=""/>
            <a href="{{ doc.url }}">{{ doc.name }}</a>
            {%- if doc.content_length %}
              ({{ doc.content_length|filesize }})
            {%- endif %}
          </span>
        </p>
      {% endfor %}
    {% endfor %}
  {%- endcall %}
{% endblock %}
